import React, { Fragment, useRef, useState } from 'react';

import { useDB, useFind } from 'react-pouchdb';

import '../../theme/get-location.css';
import {
  IonButton,
  IonButtons,
  IonContent,
  IonIcon,
  IonModal,
  IonTitle,
  IonToolbar,
} from '@ionic/react';
import {
  recordingOutline,
  recording,
  closeCircle,
  pauseCircle,
  stop,
} from 'ionicons/icons';
import {
  startBackgroundGeolocation,
  stopBackgroundGeolocation,
} from '../../lib/background-geolocation';
import { deleteCurrent, saveCurrent } from '../../db/gpx';
import { enterAnimation, leaveAnimation } from '../../lib/animation';

const GpxRecord: React.FC<{}> = () => {
  const db = useDB();

  const [isRecording, setIsRecording] = useState(false);

  const [watcher_id, setWatcher_id] = useState();

  const gpxes = useFind({
    selector: {
      type: 'gpx',
      subtype: 'current',
    },
  });

  const hasCurrentTrack = gpxes.length > 0;

  console.log(
    `GpxRecord, hasCurrentTrack:${hasCurrentTrack}, gpxes: ${JSON.stringify(
      gpxes
    )}`
  );

  const modal = useRef<HTMLIonModalElement>(null);

  const dismiss = () => {
    modal.current?.dismiss();
  };

  const startRecording = () => {
    startBackgroundGeolocation(db).then((result) => {
      setWatcher_id(result);
    });
    setIsRecording(true);
    dismiss();
  };

  const pauseRecording = () => {
    if (isRecording) {
      stopBackgroundGeolocation(watcher_id);
    }
    setIsRecording(false);
    dismiss();
  };

  const stopRecording = () => {
    saveCurrent(db);
    pauseRecording();
  };

  const deleteRecording = () => {
    deleteCurrent(db);
    pauseRecording();
  };


  return (
    <Fragment>
      <IonButton id='open-RecordDialog'>
        {isRecording && (
          <IonIcon slot='icon-only' icon={recording} style={{ color: 'red' }} />
        )}
        {!isRecording && <IonIcon slot='icon-only' icon={recordingOutline} />}
      </IonButton>
      <IonModal
        ref={modal}
        trigger='open-RecordDialog'
        enterAnimation={enterAnimation}
        leaveAnimation={leaveAnimation}
      >
        <IonToolbar>
          <IonTitle>Track recording</IonTitle>
          <IonButtons slot='end'>
            <IonButton onClick={() => dismiss()}>Close</IonButton>
          </IonButtons>
        </IonToolbar>
        <IonContent>
          {!isRecording && (
            <IonButton
              expand='block'
              color='primary'
              size='large'
              onClick={startRecording}
            >
              <IonIcon slot='start' icon={recording}></IonIcon>
              {hasCurrentTrack ? (
                <span>Resume recording</span>
              ) : (
                <span>Start recording</span>
              )}
            </IonButton>
          )}
          {isRecording && (
            <IonButton
              expand='block'
              color='primary'
              size='large'
              onClick={pauseRecording}
            >
              <IonIcon slot='start' icon={pauseCircle}></IonIcon>Pause
            </IonButton>
          )}
          {hasCurrentTrack && (
            <>
              <IonButton
                expand='block'
                color='danger'
                size='large'
                onClick={stopRecording}
              >
                <IonIcon slot='start' icon={stop}></IonIcon>Stop recording
                <br />
                (and save track)
              </IonButton>
              <IonButton
                expand='block'
                color='danger'
                size='large'
                onClick={deleteRecording}
              >
                <IonIcon slot='start' icon={closeCircle}></IonIcon>Cancel
                recording
                <br />
                (and clear track)
              </IonButton>
            </>
          )}
        </IonContent>
      </IonModal>
    </Fragment>
  );
};

export default GpxRecord;
